@@ -3,20 +3,16 @@ use strict;
 use warnings;
 use if $] >= 5.019, 'deprecate';
 
-$CGI::Fast::VERSION='2.14';
+$CGI::Fast::VERSION='2.17';
 
 use CGI;
 use CGI::Carp;
 use FCGI;
-# use vars works like "our", but is compatible with older Perls.
-use vars qw(
-    @ISA
-    $ignore
-);
-@ISA = ('CGI');
+
+our @ISA = ('CGI');
 
 # workaround for known bug in libfcgi
-while (($ignore) = each %ENV) { }
+while (() = each %ENV) { }
 
 # override the initialization behavior so that
 # state is NOT maintained between invocations
@@ -26,7 +22,7 @@ sub save_request {
 
 # If ENV{FCGI_SOCKET_PATH} is specified, we maintain a FCGI Request handle
 # in this package variable.
-use vars qw($Ext_Request $socket $socket_perm $queue);
+our ($Ext_Request, $socket, $socket_perm, $queue);
 
 sub import {
     my ($package,@import) = @_;
@@ -87,15 +83,35 @@ sub _create_fcgi_request {
     }
 
     sub new {
-        my ($self, $initializer, @param) = @_;
 
-        if ( ! defined $initializer ) {
+		#
+		# the interface to the ->new method is unfortunately somewhat
+		# overloaded as it can be passed:
+		#
+		#         nothing
+		#         an upload hook, "something", 0
+		#         an initializer, an upload hook, "something", 0
+		#
+		# these then get passed through to the SUPER class (CGI.pm) that
+		# also has a constructor that can take various order of args
+		#
+        my ($self, @args) = @_;
+
+        if (
+			! $args[0]
+			|| (
+				ref( $args[0] )
+				&& UNIVERSAL::isa( $args[0],'CODE' )
+				&& ! $args[3]
+			)
+		) {
             $Ext_Request ||= _create_fcgi_request( $in_fh,$out_fh,$err_fh );
-            return undef unless $Ext_Request->Accept >= 0;
+			my $accept = $Ext_Request->Accept;
+            return undef unless ( defined $accept && $accept >= 0 );
         }
         CGI->_reset_globals;
         $self->_setup_symbols(@CGI::SAVED_SYMBOLS) if @CGI::SAVED_SYMBOLS;
-        return $CGI::Q = $self->SUPER::new($initializer, @param);
+        return $CGI::Q = $self->SUPER::new(@args);
     }
 }
 
@@ -308,14 +324,12 @@ maintained by Lee Johnson
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+=head1 BUGS
+
 Address bug reports and comments to:
 
     https://github.com/leejo/cgi-fast
 
-=head1 BUGS
-
-This section intentionally left blank.
-
 =head1 SEE ALSO
 
 L<CGI::Carp>, L<CGI>
